@@ -2,7 +2,20 @@
  * Push badges to backpack.
  */
 function addtobackpack(event, args) {
-    OpenBadges.issue([args.assertion], function(errors, successes) { });
+    var badgetable = Y.one('#issued-badge-table');
+    var errordiv = Y.one('#addtobackpack-error');
+    var errortext = M.util.get_string('error:backpackproblem', 'badges');
+    var errorhtml = '<div id="addtobackpack-error" class="box boxaligncenter notifyproblem">' + errortext + '</div>';
+
+    if (typeof OpenBadges !== 'undefined') {
+        OpenBadges.issue([args.assertion], function(errors, successes) { });
+    } else {
+        // Add error div if it doesn't exist yet.
+        if (!errordiv) {
+            var badgerror = Y.Node.create(errorhtml);
+            badgetable.insert(badgerror, 'before');
+        }
+    }
 }
 
 /**
