@@ -119,6 +119,23 @@ class cache_phpunit_tests extends advanced_testcase {
         }
     }
 
+    /**
+     * Tests for cache keys that would break on windows.
+     */
+    public function test_windows_nasty_keys() {
+        $instance = cache_config_phpunittest::instance();
+        $instance->phpunit_add_definition('phpunit/windowskeytest', array(
+            'mode' => cache_store::MODE_APPLICATION,
+            'component' => 'phpunit',
+            'area' => 'windowskeytest',
+            'simplekeys' => true,
+            'simpledata' => true
+        ));
+        $cache = cache::make('phpunit', 'windowskeytest');
+        $this->assertTrue($cache->set('contest', 'test data 1'));
+        $this->assertEquals('test data 1', $cache->get('contest'));
+    }
+
     /**
      * Tests the default application cache
      */
@@ -191,42 +208,44 @@ class cache_phpunit_tests extends advanced_testcase {
      * @param cache_loader $cache
      */
     protected function run_on_cache(cache_loader $cache) {
-        $key = 'testkey';
-        $datascalar = 'test data';
-        $dataarray = array('test' => 'data', 'part' => 'two');
+        $key = 'contestkey';
+        $datascalars = array('test data', null);
+        $dataarray = array('contest' => 'data', 'part' => 'two');
         $dataobject = (object)$dataarray;
 
-        $this->assertTrue($cache->purge());
+        foreach ($datascalars as $datascalar) {
+            $this->assertTrue($cache->purge());
 
-        // Check all read methods.
-        $this->assertFalse($cache->get($key));
-        $this->assertFalse($cache->has($key));
-        $result = $cache->get_many(array($key));
-        $this->assertCount(1, $result);
-        $this->assertFalse(reset($result));
-        $this->assertFalse($cache->has_any(array($key)));
-        $this->assertFalse($cache->has_all(array($key)));
+            // Check all read methods.
+            $this->assertFalse($cache->get($key));
+            $this->assertFalse($cache->has($key));
+            $result = $cache->get_many(array($key));
+            $this->assertCount(1, $result);
+            $this->assertFalse(reset($result));
+            $this->assertFalse($cache->has_any(array($key)));
+            $this->assertFalse($cache->has_all(array($key)));
 
-        // Set the data.
-        $this->assertTrue($cache->set($key, $datascalar));
-        // Setting it more than once should be permitted.
-        $this->assertTrue($cache->set($key, $datascalar));
+            // Set the data.
+            $this->assertTrue($cache->set($key, $datascalar));
+            // Setting it more than once should be permitted.
+            $this->assertTrue($cache->set($key, $datascalar));
 
-        // Recheck the read methods.
-        $this->assertEquals($datascalar, $cache->get($key));
-        $this->assertTrue($cache->has($key));
-        $result = $cache->get_many(array($key));
-        $this->assertCount(1, $result);
-        $this->assertEquals($datascalar, reset($result));
-        $this->assertTrue($cache->has_any(array($key)));
-        $this->assertTrue($cache->has_all(array($key)));
+            // Recheck the read methods.
+            $this->assertEquals($datascalar, $cache->get($key));
+            $this->assertTrue($cache->has($key));
+            $result = $cache->get_many(array($key));
+            $this->assertCount(1, $result);
+            $this->assertEquals($datascalar, reset($result));
+            $this->assertTrue($cache->has_any(array($key)));
+            $this->assertTrue($cache->has_all(array($key)));
 
-        // Delete it.
-        $this->assertTrue($cache->delete($key));
+            // Delete it.
+            $this->assertTrue($cache->delete($key));
 
-        // Check its gone.
-        $this->assertFalse($cache->get($key));
-        $this->assertFalse($cache->has($key));
+            // Check its gone.
+            $this->assertFalse($cache->get($key));
+            $this->assertFalse($cache->has($key));
+        }
 
         // Test arrays.
         $this->assertTrue($cache->set($key, $dataarray));
@@ -261,11 +280,13 @@ class cache_phpunit_tests extends advanced_testcase {
         }
 
         // Test set many.
-        $cache->set_many(array('key1' => 'data1', 'key2' => 'data2'));
+        $cache->set_many(array('key1' => 'data1', 'key2' => 'data2', 'key3' => null));
         $this->assertEquals('data1', $cache->get('key1'));
         $this->assertEquals('data2', $cache->get('key2'));
+        $this->assertEquals(null, $cache->get('key3'));
         $this->assertTrue($cache->delete('key1'));
         $this->assertTrue($cache->delete('key2'));
+        $this->assertTrue($cache->delete('key3'));
 
         $cache->set_many(array(
             'key1' => array(1, 2, 3),
@@ -282,14 +303,17 @@ class cache_phpunit_tests extends advanced_testcase {
         // Test delete many.
         $this->assertTrue($cache->set('key1', 'data1'));
         $this->assertTrue($cache->set('key2', 'data2'));
+        $this->assertTrue($cache->set('key3', null));
 
         $this->assertEquals('data1', $cache->get('key1'));
         $this->assertEquals('data2', $cache->get('key2'));
+        $this->assertEquals(null, $cache->get('key3'));
 
-        $this->assertEquals(2, $cache->delete_many(array('key1', 'key2')));
+        $this->assertEquals(3, $cache->delete_many(array('key1', 'key2', 'key3')));
 
         $this->assertFalse($cache->get('key1'));
         $this->assertFalse($cache->get('key2'));
+        $this->assertFalse($cache->get('key3'));
 
         // Quick reference test.
         $obj = new stdClass;
@@ -843,7 +867,7 @@ class cache_phpunit_tests extends advanced_testcase {
         // OK data added, data invalidated, and invalidation time has been set.
         // Now we need to manually add back the data and adjust the invalidation time.
         $hash = md5(cache_store::MODE_APPLICATION.'/phpunit/eventinvalidationtest/'.$CFG->wwwroot.'phpunit');
-        $timefile = $CFG->dataroot."/cache/cachestore_file/default_application/phpunit_eventinvalidationtest/las/lastinvalidation-$hash.cache";
+        $timefile = $CFG->dataroot."/cache/cachestore_file/default_application/phpunit_eventinvalidationtest/las-cache/lastinvalidation-$hash.cache";
         // Make sure the file is correct.
         $this->assertTrue(file_exists($timefile));
         $timecont = serialize(cache::now() - 60); // Back 60sec in the past to force it to re-invalidate.
@@ -851,7 +875,7 @@ class cache_phpunit_tests extends advanced_testcase {
         file_put_contents($timefile, $timecont);
         $this->assertTrue(file_exists($timefile));
 
-        $datafile = $CFG->dataroot."/cache/cachestore_file/default_application/phpunit_eventinvalidationtest/tes/testkey1-$hash.cache";
+        $datafile = $CFG->dataroot."/cache/cachestore_file/default_application/phpunit_eventinvalidationtest/tes-cache/testkey1-$hash.cache";
         $datacont = serialize("test data 1");
         make_writable_directory(dirname($datafile));
         file_put_contents($datafile, $datacont);
@@ -1102,9 +1126,9 @@ class cache_phpunit_tests extends advanced_testcase {
     }
 
     /**
-     * Test that multiple loaders work ok.
+     * Test that multiple application loaders work ok.
      */
-    public function test_multiple_loaders() {
+    public function test_multiple_application_loaders() {
         $instance = cache_config_phpunittest::instance(true);
         $instance->phpunit_add_file_store('phpunittest1');
         $instance->phpunit_add_file_store('phpunittest2');
@@ -1149,6 +1173,93 @@ class cache_phpunit_tests extends advanced_testcase {
         $this->assertFalse($result['a']);
         $this->assertEquals('B', $result['b']);
         $this->assertFalse($result['c']);
+
+        // Test non-recursive deletes.
+        $this->assertTrue($cache->set('test', 'test'));
+        $this->assertSame('test', $cache->get('test'));
+        $this->assertTrue($cache->delete('test', false));
+        // We should still have it on a deeper loader.
+        $this->assertSame('test', $cache->get('test'));
+        // Test non-recusive with many functions.
+        $this->assertSame(3, $cache->set_many(array(
+            'one' => 'one',
+            'two' => 'two',
+            'three' => 'three'
+        )));
+        $this->assertSame('one', $cache->get('one'));
+        $this->assertSame(array('two' => 'two', 'three' => 'three'), $cache->get_many(array('two', 'three')));
+        $this->assertSame(3, $cache->delete_many(array('one', 'two', 'three'), false));
+        $this->assertSame('one', $cache->get('one'));
+        $this->assertSame(array('two' => 'two', 'three' => 'three'), $cache->get_many(array('two', 'three')));
+    }
+
+    /**
+     * Test that multiple application loaders work ok.
+     */
+    public function test_multiple_session_loaders() {
+        /* @var cache_config_phpunittest $instance */
+        $instance = cache_config_phpunittest::instance(true);
+        $instance->phpunit_add_session_store('phpunittest1');
+        $instance->phpunit_add_session_store('phpunittest2');
+        $instance->phpunit_add_definition('phpunit/multi_loader', array(
+            'mode' => cache_store::MODE_SESSION,
+            'component' => 'phpunit',
+            'area' => 'multi_loader'
+        ));
+        $instance->phpunit_add_definition_mapping('phpunit/multi_loader', 'phpunittest1', 3);
+        $instance->phpunit_add_definition_mapping('phpunit/multi_loader', 'phpunittest2', 2);
+
+        $cache = cache::make('phpunit', 'multi_loader');
+        $this->assertInstanceOf('cache_session', $cache);
+        $this->assertFalse($cache->get('test'));
+        $this->assertTrue($cache->set('test', 'test'));
+        $this->assertEquals('test', $cache->get('test'));
+        $this->assertTrue($cache->delete('test'));
+        $this->assertFalse($cache->get('test'));
+        $this->assertTrue($cache->set('test', 'test'));
+        $this->assertTrue($cache->purge());
+        $this->assertFalse($cache->get('test'));
+
+        // Test the many commands.
+        $this->assertEquals(3, $cache->set_many(array('a' => 'A', 'b' => 'B', 'c' => 'C')));
+        $result = $cache->get_many(array('a', 'b', 'c'));
+        $this->assertInternalType('array', $result);
+        $this->assertCount(3, $result);
+        $this->assertArrayHasKey('a', $result);
+        $this->assertArrayHasKey('b', $result);
+        $this->assertArrayHasKey('c', $result);
+        $this->assertEquals('A', $result['a']);
+        $this->assertEquals('B', $result['b']);
+        $this->assertEquals('C', $result['c']);
+        $this->assertEquals($result, $cache->get_many(array('a', 'b', 'c')));
+        $this->assertEquals(2, $cache->delete_many(array('a', 'c')));
+        $result = $cache->get_many(array('a', 'b', 'c'));
+        $this->assertInternalType('array', $result);
+        $this->assertCount(3, $result);
+        $this->assertArrayHasKey('a', $result);
+        $this->assertArrayHasKey('b', $result);
+        $this->assertArrayHasKey('c', $result);
+        $this->assertFalse($result['a']);
+        $this->assertEquals('B', $result['b']);
+        $this->assertFalse($result['c']);
+
+        // Test non-recursive deletes.
+        $this->assertTrue($cache->set('test', 'test'));
+        $this->assertSame('test', $cache->get('test'));
+        $this->assertTrue($cache->delete('test', false));
+        // We should still have it on a deeper loader.
+        $this->assertSame('test', $cache->get('test'));
+        // Test non-recusive with many functions.
+        $this->assertSame(3, $cache->set_many(array(
+            'one' => 'one',
+            'two' => 'two',
+            'three' => 'three'
+        )));
+        $this->assertSame('one', $cache->get('one'));
+        $this->assertSame(array('two' => 'two', 'three' => 'three'), $cache->get_many(array('two', 'three')));
+        $this->assertSame(3, $cache->delete_many(array('one', 'two', 'three'), false));
+        $this->assertSame('one', $cache->get('one'));
+        $this->assertSame(array('two' => 'two', 'three' => 'three'), $cache->get_many(array('two', 'three')));
     }
 
     /**
@@ -1374,4 +1485,54 @@ class cache_phpunit_tests extends advanced_testcase {
             $this->assertContains('Identifier required for cache has not been provided', $ex->getMessage());
         }
     }
-}
+
+    /**
+     * Test that the default stores all support searching.
+     */
+    public function test_defaults_support_searching() {
+        $instance = cache_config_phpunittest::instance(true);
+        $instance->phpunit_add_definition('phpunit/search1', array(
+            'mode' => cache_store::MODE_APPLICATION,
+            'component' => 'phpunit',
+            'area' => 'search1',
+            'requiresearchable' => true
+        ));
+        $instance->phpunit_add_definition('phpunit/search2', array(
+            'mode' => cache_store::MODE_SESSION,
+            'component' => 'phpunit',
+            'area' => 'search2',
+            'requiresearchable' => true
+        ));
+        $instance->phpunit_add_definition('phpunit/search3', array(
+            'mode' => cache_store::MODE_REQUEST,
+            'component' => 'phpunit',
+            'area' => 'search3',
+            'requiresearchable' => true
+        ));
+        $factory = cache_factory::instance();
+
+        // Test application cache is searchable.
+        $definition = $factory->create_definition('phpunit', 'search1');
+        $this->assertInstanceOf('cache_definition', $definition);
+        $this->assertEquals(cache_store::IS_SEARCHABLE, $definition->get_requirements_bin() & cache_store::IS_SEARCHABLE);
+        $cache = $factory->create_cache($definition);
+        $this->assertInstanceOf('cache_application', $cache);
+        $this->assertArrayHasKey('cache_is_searchable', $cache->phpunit_get_store_implements());
+
+        // Test session cache is searchable.
+        $definition = $factory->create_definition('phpunit', 'search2');
+        $this->assertInstanceOf('cache_definition', $definition);
+        $this->assertEquals(cache_store::IS_SEARCHABLE, $definition->get_requirements_bin() & cache_store::IS_SEARCHABLE);
+        $cache = $factory->create_cache($definition);
+        $this->assertInstanceOf('cache_session', $cache);
+        $this->assertArrayHasKey('cache_is_searchable', $cache->phpunit_get_store_implements());
+
+        // Test request cache is searchable.
+        $definition = $factory->create_definition('phpunit', 'search3');
+        $this->assertInstanceOf('cache_definition', $definition);
+        $this->assertEquals(cache_store::IS_SEARCHABLE, $definition->get_requirements_bin() & cache_store::IS_SEARCHABLE);
+        $cache = $factory->create_cache($definition);
+        $this->assertInstanceOf('cache_request', $cache);
+        $this->assertArrayHasKey('cache_is_searchable', $cache->phpunit_get_store_implements());
+    }
+}
\ No newline at end of file
