@@ -1,15 +1,37 @@
-<?php // $Id$
-      // editor.php - created with Moodle 1.2 (2004032000)
+<?php
 
+// This file is part of Moodle - http://moodle.org/
+//
+// Moodle is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Moodle is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
+
+/**
+ * Strings for component 'editor', language 'en', branch 'MOODLE_20_STABLE'
+ *
+ * @package   editor
+ * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
+ * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
+ */
 
 $string['about'] = 'About this editor';
 $string['absbottom'] = 'Absbottom';
-$string['middle'] = 'middle';
+$string['acteditorshhdr'] = 'Available text editors';
 $string['address'] = 'Address';
+$string['alertnoselectedtext'] = 'You must select text first!';
 $string['alignment'] = 'Alignment';
 $string['alternatetext'] = 'Alternate text';
-$string['anchorname'] = 'Anchor name';
 $string['anchorhelp'] = 'This only creates an anchor.  You still need to create a link manually.';
+$string['anchorname'] = 'Anchorname';
 $string['anchors'] = 'Anchors';
 $string['baseline'] = 'Baseline';
 $string['bold'] = 'Bold';
@@ -19,18 +41,27 @@ $string['browse'] = 'Browse';
 $string['cancel'] = 'Cancel';
 $string['cellpadding'] = 'Cellpadding';
 $string['cellspacing'] = 'Cellspacing';
-$string['choosechar'] = 'Choose Character';
-$string['chooseicon'] = 'Choose an icon to insert';
 $string['close'] = 'Close';
+$string['closeafterreplace'] = 'Close after replace';
 $string['cols'] = 'Cols';
+$string['configeditorplugins'] = 'Please choose the editor plugins you wish to use and arrange them in recommended order.';
 $string['copy'] = 'Copy selection';
 $string['createanchor'] = 'Create anchor';
 $string['createfolder'] = 'Create folder';
 $string['createlink'] = 'Insert Web Link';
 $string['cut'] = 'Cut selection';
+$string['cutpastemozilla'] = 'Unfortunately, you cannot currently use normal keyboard shortcuts (or even the Paste button) for pasting text into this online editor.  This is because of a security feature that is built into some versions of Mozilla and Firefox browsers.<br /><br />There are three known workarounds you can try: <br /> (1) Instead of CTRL-v, use SHIFT-Insert <br /> (2) Use the Edit->Paste menu in your browser <br /> (3) Change the preferences in your browser by editing the user.js file. <br /><br />Click the OK button below for more help, or Cancel to return to the editor';
 $string['delete'] = 'Delete';
+$string['editordeleteconfirm'] = 'You are about to completely delete the editor plugin \'{$a}\'. This will completely delete everything in the database associated with this plugin. Are you SURE you want to continue?';
+$string['editors'] = 'Text editors';
+$string['editorsettings'] = 'Manage editors';
+$string['enterurlfirst'] = 'You have to enter an URL first';
 $string['filebrowser'] = 'File Browser';
+$string['findwhat'] = 'Find';
+$string['fontname'] = 'Fontname';
+$string['fontsize'] = 'Font-size';
 $string['forecolor'] = 'Font Color';
+$string['formatblock'] = 'Format';
 $string['fullscreen'] = 'Fullscreen editor';
 $string['heading'] = 'Heading';
 $string['height'] = 'Height';
@@ -38,6 +69,8 @@ $string['hilitecolor'] = 'Background Color';
 $string['horizontal'] = 'Horizontal';
 $string['horizontalrule'] = 'Horizontal Rule';
 $string['htmlmode'] = 'Toggle HTML Source';
+$string['choosechar'] = 'Choose Character';
+$string['chooseicon'] = 'Choose an icon to insert';
 $string['imageurl'] = 'Image URL';
 $string['indent'] = 'Increase Indent';
 $string['insertchar'] = 'Insert Special Character';
@@ -46,10 +79,12 @@ $string['insertlink'] = 'Insert Link';
 $string['insertsmile'] = 'Insert Smiley';
 $string['inserttable'] = 'Insert Table';
 $string['italic'] = 'Italic';
+$string['itemsreplaced'] = 'items replaced!';
 $string['justifycenter'] = 'Justify Center';
 $string['justifyfull'] = 'Justify Full';
 $string['justifyleft'] = 'Justify Left';
 $string['justifyright'] = 'Justify Right';
+$string['lang'] = 'Lang';
 $string['layout'] = 'Layout';
 $string['left'] = 'Left';
 $string['lefttoright'] = 'Direction from left to right';
@@ -62,29 +97,40 @@ $string['linktargetself'] = 'Same frame';
 $string['linktargettop'] = 'Same window';
 $string['linktitle'] = 'Title';
 $string['linkurl'] = 'URL';
+$string['matchcase'] = 'Match case';
 $string['middle'] = 'Middle';
 $string['minimize'] = 'Minimize editor';
 $string['move'] = 'Move';
+$string['multi'] = 'Multi';
+$string['mustenterurl'] = 'You must enter the URL';
+$string['nolink'] = 'Prevent automatic linking';
 $string['normal'] = 'Normal';
 $string['notimage'] = 'Selected file is not an image. Please select an image!';
 $string['notset'] = 'Not Set';
 $string['ok'] = 'OK';
+$string['options'] = 'Options';
 $string['orderedlist'] = 'Ordered List';
 $string['outdent'] = 'Decrease Indent';
 $string['paste'] = 'Paste from clipboard';
 $string['path'] = 'Path';
 $string['percent'] = 'Percent';
 $string['pixels'] = 'Pixels';
+$string['pleaseenteralt'] = 'Please enter the alternate text';
 $string['popupeditor'] = 'Enlarge Editor';
 $string['preformatted'] = 'Preformatted';
 $string['preview'] = 'Preview';
 $string['properties'] = 'Properties';
 $string['redo'] = 'Redo your last action';
-$string['rename'] = 'Rename';
+$string['regularexpressions'] = 'Use regular expressions';
 $string['removelink'] = 'Remove link';
+$string['rename'] = 'Rename';
+$string['replaceall'] = 'Replace all';
+$string['replacewith'] = 'Replace';
 $string['right'] = 'Right';
 $string['righttoleft'] = 'Direction from right to left';
 $string['rows'] = 'Rows';
+$string['searchandreplace'] = 'Search and replace';
+$string['searchnotfound'] = 'Search string Not Found!';
 $string['selectcolor'] = 'Select color';
 $string['selection'] = 'Selection';
 $string['showhelp'] = 'Help using editor';
@@ -106,12 +152,3 @@ $string['vertical'] = 'Vertical';
 $string['width'] = 'Width';
 $string['wordclean'] = 'Clean Word HTML';
 $string['zip'] = 'Zip';
-$string['createanchor'] = 'Create anchor';
-$string['anchorname'] = 'Anchorname';
-$string['fontname'] = 'Fontname';
-$string['fontsize'] = 'Font-size';
-$string['formatblock'] = 'Format';
-$string['nolink'] = 'Prevent glossary link';
-$string['alertnoselectedtext'] = 'You must select text first!';
-
-?>
