@@ -1,31 +1,49 @@
 <?php
-    if (isset($userdata->status)) {
-        if ($userdata->status == '') {
-            $userdata->entry = 'ab-initio';
+// This file is part of Moodle - http://moodle.org/
+//
+// Moodle is free software: you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation, either version 3 of the License, or
+// (at your option) any later version.
+//
+// Moodle is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
+
+require_once($CFG->dirroot.'/mod/scorm/locallib.php');
+
+if (isset($userdata->status)) {
+    if ($userdata->status == '') {
+        $userdata->entry = 'ab-initio';
+    } else {
+        if (isset($userdata->{'cmi.core.exit'}) && ($userdata->{'cmi.core.exit'} == 'suspend')) {
+            $userdata->entry = 'resume';
         } else {
-            if (isset($userdata->{'cmi.core.exit'}) && ($userdata->{'cmi.core.exit'} == 'suspend')) {
-                $userdata->entry = 'resume';
-            } else {
-                $userdata->entry = '';
-            }
+            $userdata->entry = '';
         }
     }
+}
+if (!isset($currentorg)) {
+    $currentorg = '';
+}
 ?>
 //
 // SCORM 1.2 API Implementation
 //
 function SCORMapi1_2() {
     // Standard Data Type Definition
-    CMIString256 = '^.{0,255}$';
-    CMIString4096 = '^.{0,4096}$';
-    //CMIString256 = '^[\.|\\n|\\r]{0,255}$';
-    //CMIString4096 = '^[\.|\\n|\\r]{0,4095}$';
+    CMIString256 = '^[\\u0000-\\uffff]{0,255}$';
+    CMIString4096 = '^[\\u0000-\\uffff]{0,4096}$';
     CMITime = '^([0-2]{1}[0-9]{1}):([0-5]{1}[0-9]{1}):([0-5]{1}[0-9]{1})(\.[0-9]{1,2})?$';
     CMITimespan = '^([0-9]{2,4}):([0-9]{2}):([0-9]{2})(\.[0-9]{1,2})?$';
     CMIInteger = '^\\d+$';
     CMISInteger = '^-?([0-9]+)$';
     CMIDecimal = '^-?([0-9]{0,3})(\.[0-9]{1,2})?$';
-    CMIIdentifier = '^\\w{1,255}$';
+    CMIIdentifier = '^[\\u0021-\\u007E]{0,255}$';
     CMIFeedback = CMIString256; // This must be redefined
     CMIIndex = '[._](\\d+).';
     // Vocabulary Data Type Definition
@@ -36,13 +54,15 @@ function SCORMapi1_2() {
     CMIResult = '^correct$|^wrong$|^unanticipated$|^neutral$|^([0-9]{0,3})?(\.[0-9]{1,2})?$';
     NAVEvent = '^previous$|^continue$';
     // Children lists
-    cmi_children = 'core, suspend_data, launch_data, comments, objectives, student_data, student_preference, interactions';
-    core_children = 'student_id, student_name, lesson_location, credit, lesson_status, entry, score, total_time, lesson_mode, exit, session_time';
-    score_children = 'raw, min, max';
-    objectives_children = 'id, score, status';
-    student_data_children = 'mastery_score, max_time_allowed, time_limit_action';
-    student_preference_children = 'audio, language, speed, text';
-    interactions_children = 'id, objectives, time, type, correct_responses, weighting, student_response, result, latency';
+    cmi_children = 'core,suspend_data,launch_data,comments,objectives,student_data,student_preference,interactions';
+    core_children = 'student_id,student_name,lesson_location,credit,lesson_status,entry,score,total_time,lesson_mode,exit,session_time';
+    score_children = 'raw,min,max';
+    comments_children = 'content,location,time';
+    objectives_children = 'id,score,status';
+    correct_responses_children = 'pattern';
+    student_data_children = 'mastery_score,max_time_allowed,time_limit_action';
+    student_preference_children = 'audio,language,speed,text';
+    interactions_children = 'id,objectives,time,type,correct_responses,weighting,student_response,result,latency';
     // Data ranges
     score_range = '0#100';
     audio_range = '-1#100';
@@ -55,7 +75,7 @@ function SCORMapi1_2() {
         'cmi._version':{'defaultvalue':'3.4', 'mod':'r', 'writeerror':'402'},
         'cmi.core._children':{'defaultvalue':core_children, 'mod':'r', 'writeerror':'402'},
         'cmi.core.student_id':{'defaultvalue':'<?php echo $userdata->student_id ?>', 'mod':'r', 'writeerror':'403'},
-        'cmi.core.student_name':{'defaultvalue':'<?php echo addslashes($userdata->student_name) ?>', 'mod':'r', 'writeerror':'403'},
+        'cmi.core.student_name':{'defaultvalue':'<?php echo $userdata->student_name ?>', 'mod':'r', 'writeerror':'403'},
         'cmi.core.lesson_location':{'defaultvalue':'<?php echo isset($userdata->{'cmi.core.lesson_location'})?$userdata->{'cmi.core.lesson_location'}:'' ?>', 'format':CMIString256, 'mod':'rw', 'writeerror':'405'},
         'cmi.core.credit':{'defaultvalue':'<?php echo $userdata->credit ?>', 'mod':'r', 'writeerror':'403'},
         'cmi.core.lesson_status':{'defaultvalue':'<?php echo isset($userdata->{'cmi.core.lesson_status'})?$userdata->{'cmi.core.lesson_status'}:'' ?>', 'format':CMIStatus, 'mod':'rw', 'writeerror':'405'},
@@ -71,6 +91,12 @@ function SCORMapi1_2() {
         'cmi.suspend_data':{'defaultvalue':'<?php echo isset($userdata->{'cmi.suspend_data'})?$userdata->{'cmi.suspend_data'}:'' ?>', 'format':CMIString4096, 'mod':'rw', 'writeerror':'405'},
         'cmi.launch_data':{'defaultvalue':'<?php echo isset($userdata->datafromlms)?$userdata->datafromlms:'' ?>', 'mod':'r', 'writeerror':'403'},
         'cmi.comments':{'defaultvalue':'<?php echo isset($userdata->{'cmi.comments'})?$userdata->{'cmi.comments'}:'' ?>', 'format':CMIString4096, 'mod':'rw', 'writeerror':'405'},
+        // deprecated evaluation attributes
+        'cmi.evaluation.comments._count':{'defaultvalue':'0', 'mod':'r', 'writeerror':'402'},
+        'cmi.evaluation.comments._children':{'defaultvalue':comments_children, 'mod':'r', 'writeerror':'402'},
+        'cmi.evaluation.comments.n.content':{'defaultvalue':'', 'pattern':CMIIndex, 'format':CMIString256, 'mod':'rw', 'writeerror':'405'},
+        'cmi.evaluation.comments.n.location':{'defaultvalue':'', 'pattern':CMIIndex, 'format':CMIString256, 'mod':'rw', 'writeerror':'405'},
+        'cmi.evaluation.comments.n.time':{'defaultvalue':'', 'pattern':CMIIndex, 'format':CMITime, 'mod':'rw', 'writeerror':'405'},
         'cmi.comments_from_lms':{'mod':'r', 'writeerror':'403'},
         'cmi.objectives._children':{'defaultvalue':objectives_children, 'mod':'r', 'writeerror':'402'},
         'cmi.objectives._count':{'mod':'r', 'defaultvalue':'0', 'writeerror':'402'},
@@ -114,6 +140,9 @@ function SCORMapi1_2() {
         cmi.student_data = new Object();
         cmi.student_preference = new Object();
         cmi.interactions = new Object();
+        // deprecated evaluation attributes
+        cmi.evaluation = new Object();
+        cmi.evaluation.comments = new Object();
 
     // Navigation Object
     var nav = new Object();
@@ -129,35 +158,15 @@ function SCORMapi1_2() {
     }
 
 <?php
-    $count = 0;
-    $objectives = '';
-    foreach($userdata as $element => $value){
-        if (substr($element,0,14) == 'cmi.objectives') {
-            preg_match('/\.(\d+)\./',$element,$matches);
-            $element = preg_replace('/\.(\d+)\./',".\$1.",$element);
-            if ($matches[1] == $count) {
-                $count++;
-                $end = strpos($element,$matches[1])+strlen($matches[1]);
-                $subelement = substr($element,0,$end);
-                echo '    '.$subelement." = new Object();\n";
-                echo '    '.$subelement.".score = new Object();\n";
-                echo '    '.$subelement.".score._children = score_children;\n";
-                echo '    '.$subelement.".score.raw = '';\n";
-                echo '    '.$subelement.".score.min = '';\n";
-                echo '    '.$subelement.".score.max = '';\n";
-            }
-            echo '    '.$element.' = \''.$value."';\n";
-        }
-    }
-    if ($count > 0) {
-        echo '    cmi.objectives._count = '.$count.";\n";
-    }
-?>
+     // reconstitute objectives
+    scorm_reconstitute_array_element($scorm->version, $userdata, 'cmi.objectives', array('score'));
+    scorm_reconstitute_array_element($scorm->version, $userdata, 'cmi.interactions', array('objectives', 'correct_responses'));
+    ?>
 
     if (cmi.core.lesson_status == '') {
         cmi.core.lesson_status = 'not attempted';
-    } 
-    
+    }
+
     //
     // API Methods definition
     //
@@ -167,13 +176,13 @@ function SCORMapi1_2() {
         errorCode = "0";
         if (param == "") {
             if (!Initialized) {
-                <?php 
-                    if (debugging('',DEBUG_DEVELOPER)) {
-                        echo 'alert("Initialized SCORM 1.2");';
-                    }
-                ?>
                 Initialized = true;
                 errorCode = "0";
+                <?php
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("LMSInitialize", param, "", errorCode);';
+                    }
+                ?>
                 return "true";
             } else {
                 errorCode = "101";
@@ -181,51 +190,75 @@ function SCORMapi1_2() {
         } else {
             errorCode = "201";
         }
+        <?php
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("LMSInitialize", param, "", errorCode);';
+            }
+        ?>
         return "false";
     }
-    
+
     function LMSFinish (param) {
         errorCode = "0";
         if (param == "") {
             if (Initialized) {
-                <?php 
-                    if (debugging('',DEBUG_DEVELOPER)) {
-                        echo 'alert("Finished SCORM 1.2");';
-                    }
-                ?>
                 Initialized = false;
                 result = StoreData(cmi,true);
                 if (nav.event != '') {
                     if (nav.event == 'continue') {
-                        setTimeout('top.document.location=top.next;',500);
+                        setTimeout('mod_scorm_launch_next_sco();',500);
                     } else {
-                        setTimeout('top.document.location=top.prev;',500);
+                        setTimeout('mod_scorm_launch_prev_sco();',500);
                     }
                 } else {
                     if (<?php echo $scorm->auto ?> == 1) {
-                        setTimeout('top.document.location=top.next;',500);
+                        setTimeout('mod_scorm_launch_next_sco();',500);
                     }
-                }    
-                return "true";
+                }
+                <?php
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("LMSFinish", "AJAXResult", result, 0);';
+                    }
+                ?>
+                result = ('true' == result) ? 'true' : 'false';
+                errorCode = (result == 'true')? '0' : '101';
+                <?php
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("LMSFinish", "result", result, 0);';
+                        echo 'LogAPICall("LMSFinish", param, "", 0);';
+                    }
+                ?>
+                // trigger TOC update
+                var sURL = "<?php echo $CFG->wwwroot; ?>" + "/mod/scorm/prereqs.php?a=<?php echo $scorm->id ?>&scoid=<?php echo $scoid ?>&attempt=<?php echo $attempt ?>&mode=<?php echo $mode ?>&currentorg=<?php echo $currentorg ?>&sesskey=<?php echo sesskey(); ?>";
+                var callback = M.mod_scorm.connectPrereqCallback;
+                YUI().use('yui2-connection', function(Y) {
+                    Y.YUI2.util.Connect.asyncRequest('GET', sURL, callback, null);
+                });
+                return result;
             } else {
                 errorCode = "301";
             }
         } else {
             errorCode = "201";
         }
+        <?php
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("LMSFinish", param, "", errorCode);';
+            }
+        ?>
         return "false";
     }
-    
+
     function LMSGetValue (element) {
         errorCode = "0";
         if (Initialized) {
             if (element !="") {
                 expression = new RegExp(CMIIndex,'g');
-                elementmodel = element.replace(expression,'.n.');
+                elementmodel = String(element).replace(expression,'.n.');
                 if ((typeof eval('datamodel["'+elementmodel+'"]')) != "undefined") {
                     if (eval('datamodel["'+elementmodel+'"].mod') != 'w') {
-                            element = element.replace(expression, "_$1.");
-                            elementIndexes = element.split('.');
+                        element = String(element).replace(expression, "_$1.");
+                        elementIndexes = element.split('.');
                         subelement = 'cmi';
                         i = 1;
                         while ((i < elementIndexes.length) && (typeof eval(subelement) != "undefined")) {
@@ -233,9 +266,9 @@ function SCORMapi1_2() {
                         }
                             if (subelement == element) {
                             errorCode = "0";
-                            <?php 
-                                if (debugging('',DEBUG_DEVELOPER)) {
-                                    echo 'alert(element+": "+eval(element));';
+                            <?php
+                                if (scorm_debugging($scorm)) {
+                                    echo 'LogAPICall("LMSGetValue", element, eval(element), 0);';
                                 }
                             ?>
                             return eval(element);
@@ -272,15 +305,20 @@ function SCORMapi1_2() {
         } else {
             errorCode = "301";
         }
+        <?php
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("LMSGetValue", element, "", errorCode);';
+            }
+        ?>
         return "";
     }
-    
+
     function LMSSetValue (element,value) {
         errorCode = "0";
         if (Initialized) {
             if (element != "") {
                 expression = new RegExp(CMIIndex,'g');
-                elementmodel = element.replace(expression,'.n.');
+                elementmodel = String(element).replace(expression,'.n.');
                 if ((typeof eval('datamodel["'+elementmodel+'"]')) != "undefined") {
                     if (eval('datamodel["'+elementmodel+'"].mod') != 'r') {
                         expression = new RegExp(eval('datamodel["'+elementmodel+'"].format'));
@@ -300,7 +338,7 @@ function SCORMapi1_2() {
                                         }
                                         if (elementIndexes[i+1] == eval(subelement+'.'+elementIndex+'._count')) {
                                             eval(subelement+'.'+elementIndex+'._count++;');
-                                        } 
+                                        }
                                         if (elementIndexes[i+1] > eval(subelement+'.'+elementIndex+'._count')) {
                                             errorCode = "201";
                                         }
@@ -337,9 +375,9 @@ function SCORMapi1_2() {
                                     if ((value >= ranges[0]) && (value <= ranges[1])) {
                                         eval(element+'=value;');
                                         errorCode = "0";
-                                        <?php 
-                                            if (debugging('',DEBUG_DEVELOPER)) {
-                                                echo 'alert(element+":= "+value);';
+                                        <?php
+                                            if (scorm_debugging($scorm)) {
+                                                echo 'LogAPICall("LMSSetValue", element, value, errorCode);';
                                             }
                                         ?>
                                         return "true";
@@ -348,14 +386,14 @@ function SCORMapi1_2() {
                                     }
                                 } else {
                                     if (element == 'cmi.comments') {
-                                        eval(element+'=value;');
+                                        cmi.comments = cmi.comments + value;
                                     } else {
                                         eval(element+'=value;');
                                     }
                                     errorCode = "0";
-                                    <?php 
-                                        if (debugging('',DEBUG_DEVELOPER)) {
-                                            echo 'alert(element+":= "+value);';
+                                    <?php
+                                        if (scorm_debugging($scorm)) {
+                                            echo 'LogAPICall("LMSSetValue", element, value, errorCode);';
                                         }
                                     ?>
                                     return "true";
@@ -376,33 +414,67 @@ function SCORMapi1_2() {
         } else {
             errorCode = "301";
         }
+       <?php
+        if (scorm_debugging($scorm)) {
+            echo 'LogAPICall("LMSSetValue", element, value, errorCode);';
+        }
+        ?>
         return "false";
     }
-    
+
     function LMSCommit (param) {
         errorCode = "0";
         if (param == "") {
             if (Initialized) {
                 result = StoreData(cmi,false);
-                <?php 
-                    if (debugging('',DEBUG_DEVELOPER)) {
-                        echo 'alert("Data Commited");';
+                // trigger TOC update
+                var sURL = "<?php echo $CFG->wwwroot; ?>" + "/mod/scorm/prereqs.php?a=<?php echo $scorm->id ?>&scoid=<?php echo $scoid ?>&attempt=<?php echo $attempt ?>&mode=<?php echo $mode ?>&currentorg=<?php echo $currentorg ?>&sesskey=<?php echo sesskey(); ?>";
+                var callback = M.mod_scorm.connectPrereqCallback;
+                YUI().use('yui2-connection', function(Y) {
+                    Y.YUI2.util.Connect.asyncRequest('GET', sURL, callback, null);
+                });
+                <?php
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("Commit", param, "", 0);';
                     }
                 ?>
-                return "true";
+                <?php
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("LMSCommit", "AJAXResult", result, 0);';
+                    }
+                ?>
+                result = ('true' == result) ? 'true' : 'false';
+                errorCode = (result =='true')? '0' : '101';
+                <?php 
+                    if (scorm_debugging($scorm)) {
+                        echo 'LogAPICall("LMSCommit", "result", result, 0);';
+                        echo 'LogAPICall("LMSCommit", "errorCode", errorCode, 0);';
+                    }
+                ?>
+                return result;
             } else {
                 errorCode = "301";
             }
         } else {
             errorCode = "201";
         }
+        <?php
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("LMSCommit", param, "", 0);';
+            }
+        ?>
         return "false";
     }
-    
+
     function LMSGetLastError () {
+     <?php
+        if (scorm_debugging($scorm)) {
+            echo 'LogAPICall("LMSGetLastError", "", "", errorCode);';
+        }
+    ?>
         return errorCode;
     }
-    
+
     function LMSGetErrorString (param) {
         if (param != "") {
             var errorString = new Array();
@@ -417,16 +489,31 @@ function SCORMapi1_2() {
             errorString["403"] = "Element is read only";
             errorString["404"] = "Element is write only";
             errorString["405"] = "Incorrect data type";
+            <?php
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("LMSGetErrorString", param,  errorString[param], 0);';
+            }
+             ?>
             return errorString[param];
         } else {
+           <?php
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("LMSGetErrorString", param,  "No error string found!", 0);';
+            }
+             ?>
            return "";
         }
     }
-    
+
     function LMSGetDiagnostic (param) {
         if (param == "") {
             param = errorCode;
         }
+        <?php
+            if (scorm_debugging($scorm)) {
+                echo 'LogAPICall("LMSGetDiagnostic", param, param, 0);';
+            }
+        ?>
         return param;
     }
 
@@ -491,17 +578,49 @@ function SCORMapi1_2() {
             } else {
                 element = parent+'.'+property;
                 expression = new RegExp(CMIIndex,'g');
-                elementmodel = element.replace(expression,'.n.');
-                if (elementmodel != "cmi.core.session_time") {
-                    if ((typeof eval('datamodel["'+elementmodel+'"]')) != "undefined") {
-                        if (eval('datamodel["'+elementmodel+'"].mod') != 'r') {
+
+                // get the generic name for this element (e.g. convert 'cmi.interactions.1.id' to 'cmi.interactions.n.id')
+                elementmodel = String(element).replace(expression,'.n.');
+
+                // ignore the session time element
+                if (element != "cmi.core.session_time") {
+
+                    // check if this specific element is not defined in the datamodel,
+                    // but the generic element name is
+                    if ((eval('typeof datamodel["'+element+'"]')) == "undefined"
+                        && (eval('typeof datamodel["'+elementmodel+'"]')) != "undefined") {
+
+                        // add this specific element to the data model (by cloning
+                        // the generic element) so we can track changes to it
+                        eval('datamodel["'+element+'"]=CloneObj(datamodel["'+elementmodel+'"]);');
+                    }
+
+                    // check if the current element exists in the datamodel
+                    if ((typeof eval('datamodel["'+element+'"]')) != "undefined") {
+
+                        // make sure this is not a read only element
+                        if (eval('datamodel["'+element+'"].mod') != 'r') {
+
                             elementstring = '&'+underscore(element)+'='+encodeURIComponent(data[property]);
-                            if ((typeof eval('datamodel["'+elementmodel+'"].defaultvalue')) != "undefined") {
-                                if (eval('datamodel["'+elementmodel+'"].defaultvalue') != data[property]) {
+
+                            // check if the element has a default value
+                            if ((typeof eval('datamodel["'+element+'"].defaultvalue')) != "undefined") {
+
+                                // check if the default value is different from the current value
+                                if (eval('datamodel["'+element+'"].defaultvalue') != data[property]
+                                    || eval('typeof(datamodel["'+element+'"].defaultvalue)') != typeof(data[property])) {
+
+                                    // append the URI fragment to the string we plan to commit
                                     datastring += elementstring;
+
+                                    // update the element default to reflect the current committed value
+                                    eval('datamodel["'+element+'"].defaultvalue=data[property];');
                                 }
                             } else {
+                                // append the URI fragment to the string we plan to commit
                                 datastring += elementstring;
+                                // no default value for the element, so set it now
+                                eval('datamodel["'+element+'"].defaultvalue=data[property];');
                             }
                         }
                     }
@@ -511,6 +630,19 @@ function SCORMapi1_2() {
         return datastring;
     }
 
+    function CloneObj(obj){
+        if(obj == null || typeof(obj) != 'object') {
+            return obj;
+        }
+
+        var temp = new obj.constructor(); // changed (twice)
+        for(var key in obj) {
+            temp[key] = CloneObj(obj[key]);
+        }
+
+        return temp;
+    }
+
     function StoreData(data,storetotaltime) {
         if (storetotaltime) {
             if (cmi.core.lesson_status == 'not attempted') {
@@ -518,19 +650,17 @@ function SCORMapi1_2() {
             }
             if (cmi.core.lesson_mode == 'normal') {
                 if (cmi.core.credit == 'credit') {
-                    if (cmi.core.lesson_status == 'completed') {
-                        if (cmi.student_data.mastery_score != '') {
-                            if (parseFloat(cmi.core.score.raw) >= parseFloat(cmi.student_data.mastery_score)) {
-                                cmi.core.lesson_status = 'passed';
-                            } else {
-                                cmi.core.lesson_status = 'failed';
-                            }
+                    if (cmi.student_data.mastery_score !== '' && cmi.core.score.raw !== '') {
+                        if (parseFloat(cmi.core.score.raw) >= parseFloat(cmi.student_data.mastery_score)) {
+                            cmi.core.lesson_status = 'passed';
+                        } else {
+                            cmi.core.lesson_status = 'failed';
                         }
                     }
                 }
             }
             if (cmi.core.lesson_mode == 'browse') {
-                if (datamodel['cmi.core.lesson_status'].defaultvalue == '') {
+                if (datamodel['cmi.core.lesson_status'].defaultvalue == '' && cmi.core.lesson_status == 'not attempted') {
                     cmi.core.lesson_status = 'browsed';
                 }
             }
@@ -541,10 +671,11 @@ function SCORMapi1_2() {
         }
         datastring += '&attempt=<?php echo $attempt ?>';
         datastring += '&scoid=<?php echo $scoid ?>';
-        
+
         var myRequest = NewHttpReq();
-        result = DoRequest(myRequest,"<?php p($CFG->wwwroot) ?>/mod/scorm/datamodel.php","id=<?php p($id) ?>&sesskey=<?php p($USER->sesskey) ?>"+datastring);
-        results = result.split('\n');
+        //alert('going to:' + "<?php p($CFG->wwwroot) ?>/mod/scorm/datamodel.php" + "id=<?php p($id) ?>&a=<?php p($a) ?>&sesskey=<?php echo sesskey() ?>"+datastring);
+        result = DoRequest(myRequest,"<?php p($CFG->wwwroot) ?>/mod/scorm/datamodel.php","id=<?php p($id) ?>&a=<?php p($a) ?>&sesskey=<?php echo sesskey() ?>"+datastring);
+        results = String(result).split('\n');
         errorCode = results[1];
         return results[0];
     }
@@ -560,3 +691,11 @@ function SCORMapi1_2() {
 }
 
 var API = new SCORMapi1_2();
+
+<?php
+// pull in the debugging utilities
+if (scorm_debugging($scorm)) {
+    include_once($CFG->dirroot.'/mod/scorm/datamodels/debug.js.php');
+    echo 'AppendToLog("Moodle SCORM 1.2 API Loaded, Activity: '.$scorm->name.', SCO: '.$sco->identifier.'", 0);';
+}
+
